
package edu.uwec.cs.wickmr.life.command;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class GameOfLife  {
	protected int ROWS;
	protected int COLS;
	Cell grid[][];
	List observers;
	
	public GameOfLife(int rows, int cols){
		this.ROWS = rows;
		this.COLS = cols;
		this.grid = new Cell[ROWS][COLS];
		this.observers = new LinkedList();
		clearGrid(grid);
	}
	protected void clearGrid(Cell[][] g){
		for(int i =0; i < ROWS; i++){
			for(int j=0; j < COLS; j++){
				g[i][j] = new Cell(i,j);
			}
		}		
	}
	public int getRows(){
		return ROWS;
	}
	public int getCols(){
		return COLS;
	}
	public Cell cellAt(int r, int c){
		return grid[r][c];
	}
	protected void toggle(int r, int c){
		grid[r][c].toggle();
		notifyObservers();
	}
	protected boolean isAlive(int r, int c){
		return (grid[r][c].isAlive());
	}
	
	public void advance(){ // Could use Composite design pattern
		// TODO: D4 - Modify the advance() method in the GameOfLife class.
		// Instructions:
		//   1. Replace the declaration of the temporary grid "newGrid" with a List called "commands".
		//   2. Remove the "clearGrid(newGrid)" statement as it is no longer necessary.
		//   3. Modify the condition marked with the "// Die Condition" comment to read
		//         "if ( n != 2 && n != 3 )".
		//   4. Replace the then portion of that "if" with code to add a new DieCommand(grid[i][j])
		//         to the "commands" list.
		//   5. Replace the code marked with the "// Born" comment with code to add a new
		//         LiveCommand(grid[i][j]) to the "commands" list. 
		//   6. Replace the "grid = newGrid" line with a while loop that executes each command
		//         held in the commands list.  Use the "iterator()" method on "commands" to get the
		//         iterator.  Use the "hasNext()" and "next()" methods of the iterator to move through
		//         the list.  Recall that you will need to cast to "(LifeCommand)" since the "next()" 
		//         method returns an Object. (See "notifyObservers()" for help.)
		Cell[][] newGrid = new Cell[ROWS][COLS];
		clearGrid(newGrid); // Sets all grid locations to dead cells by default.
		int n =0;
		for(int i = 0; i < ROWS; i++){
			for(int j = 0; j < COLS; j++){
				n = grid[i][j].nbrOfNeighbors(this);
				if ( grid[i][j].isAlive() ){
					if ( n == 2 || n == 3 ){ // Die Condition
						newGrid[i][j].live();
					}
				}
				else {
					if ( n == 3){
						newGrid[i][j].live(); // Born
					}
				}
				
			}
		}
		grid = newGrid;
		notifyObservers();
	}

	// Add the given GameOfLifeUI to the list of observers.
	public void attach(GameOfLifeUI userInterface){
		observers.add(userInterface);
	}
	
	// Remove the given GameOfLifeUI from the list of observers.
	public void detach(GameOfLifeUI userInterface){
		observers.remove(userInterface);
	}
	
	// Inform all observers to update.
	public void notifyObservers(){
		Iterator uiItr = observers.iterator();
		while (uiItr.hasNext()){
			((GameOfLifeUI)uiItr.next()).update();
		}
	}

}
