
package edu.uwec.cs.wickmr.life.observer;

import java.util.*;
/*
 * 
 * This class represents the Life game 
 */
public class GameOfLife {
	protected int ROWS;
	protected int COLS;
	protected boolean grid[][];
	protected List observers; // Holds access to all observers
	
	public GameOfLife(int rows, int cols){
		this.ROWS = rows;
		this.COLS = cols;
		grid = new boolean[rows][cols];
		observers = new LinkedList();
		for(int i =0; i < rows; i++){
			for(int j=0; j < cols; j++){
				grid[i][j] = false;
			}
		}
	}
	public int getRows(){
		return ROWS;
	}
	public int getCols(){
		return COLS;
	}
	
	// Return true if the cell at r,c is alive.
	public boolean isAlive(int r, int c){
		return grid[r][c];
	}
	
	// Reverse the life status of the cell at location r,c
	public void toggle(int r, int c){
		grid[r][c] = !grid[r][c];
		notifyObservers();
	}
	
	// Returns the number of alive neighbors for the cell at position r,c.  The
	//  grid is finite but unbound, meaning that the neighbor relation wraps around
	//  the grid boundaries.
	protected int nbrOfNeighbors(int r, int c){
		int n = 0;
		int x = 0;
		int y = r-1;
		if ( y < 0 ){ y = ROWS-1; }
		for (int rCt = 1; rCt <= 3; rCt++){
			x = c-1;
			if ( x < 0 ){ x = COLS-1;}
			for(int cCt = 1; cCt <= 3; cCt++){
				if ( x != c || y != r ){
					if ( grid[y][x] ){
						n++;
					}
				}
				x = (x+1) % COLS;
			}
			y = (y+1)%ROWS;
		}
		return n;
	}
	
	// This method implements the rules of the Game of Life.  For each cell,
	//  we simple find the number of neighbors and then bring the cell to life
	//  if appropriate. 
	public void advance(){
		// TODO: A4 - Modify the advance() method in GameOfLife class.
		//   1. Replace the "Notify Observers" comment with a call to the "notifyObservers()" method of 
		//         this class.
		boolean[][] newGrid = new boolean[ROWS][COLS];
		int n =0;
		for(int i = 0; i < ROWS; i++){
			for(int j = 0; j < COLS; j++){
				n = nbrOfNeighbors(i,j);
				if ( grid[i][j] ){
					if ( n == 2 || n == 3 ){
						newGrid[i][j] = true;
					}
					else {
						newGrid[i][j] = false;
					}
				}
				else {
					if ( n == 3){
						newGrid[i][j] = true;
					}
					else {
						newGrid[i][j] = false;
					}
				}
				
			}
		}
		grid = newGrid;
		// Notify Observers
	}
	
	// Add the given user interface to the list of observers.
	public void attach(GameOfLifeUI userInterface){
		// TODO: A1 - Implement the attach(GameOfLifeUI) method in the GameOfLife class.
		// Instructions:
		//   1. Add the given userInterface to the observers list using the "add(Object)" method of List.
	}
	
	// Remove the given user interface from the list of observers.
	public void detach(GameOfLife userInterface){
		// TODO: A2 - Implement the detach(GameOfLifeUI) method in the GameOfLife class.
		// Instructions:
		//   1. Remove the given userInterface from the observers list using the "remove(Object)" method of List.
	}
	
	// Inform all observers to update.
	public void notifyObservers(){
		// TODO: A3 - Implement the notifyObservers() method in the GameOfLife class.
		// Instructions:
		//   1. Declare an Iterator variable (called uiItr).
		//   2. Assign uiItr an iterator over the elements of the observers list using the "iterator()" method.
		//   3. Write a while loop to move over the elements pointed to by uiItr using the "hasNext()" and
		//         "next()" method of Iterator.
		//   4. The above loop should invoke the "update()" method on each GameOfLifeUI found through uiItr 
		//         (notice type casting is required since "next()" returns an Object not a GameOfLifeUI).
	}
}
