
package edu.uwec.cs.wickmr.life.simplelife;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
/*
 * 
 * This class represents both the Life game and the UI of that game.
 */
public class GameOfLifePanel extends JPanel {
	protected int rows;
	protected int cols;
	protected int cellHeight;
	protected int cellWidth;
	protected int panelWidth;
	protected int panelHeight;
	protected boolean grid[][]; // Holds the dead and alive cells.
	
	public GameOfLifePanel(int rows, int cols, int panelWidth, int panelHeight){
		this.rows = rows;
		this.cols = cols;
		this.panelWidth = panelWidth;
		this.panelHeight = panelHeight;
		this.cellWidth = panelWidth / cols;
		this.cellHeight = panelHeight / rows;
		grid = new boolean[rows][cols];
		for(int i =0; i < rows; i++){
			for(int j=0; j < cols; j++){
				grid[i][j] = false;
			}
		}
		initialize();
	}
	public int getRows(){
		return rows;
	}
	public int getCols(){
		return cols;
	}
	protected void initialize(){
		// Set the size of the Panel to  the Width x Height
		this.setSize(cols * cellWidth, rows * cellHeight);
		this.setPreferredSize(this.getSize());
		this.setBackground(Color.black);
		// Add a listener to the mouse that will cause the life at the x,y 
		//  position of the grid to be turned on/off.
		this.addMouseListener(new MouseListener(){
			public void mouseClicked(MouseEvent e) {
			}
			public void mouseEntered(MouseEvent e) {
			}
			public void mouseExited(MouseEvent e) {
			}
			public void mousePressed(MouseEvent e) {
				toggle(e.getPoint().y / cellHeight, e.getPoint().x / cellWidth);
			}
			public void mouseReleased(MouseEvent e) {
			}});
	}
	
	// Necessary painting method which draws the lines of the grid and the live cells.
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		drawGrid(g);
		drawLives(g);
	}
	
	// Reverse the life status of the cell at location r,c
	public void toggle(int r, int c){
		grid[r][c] = !grid[r][c];
		repaint();
	}
	
	// Draws the lines of the grid.
	protected void drawGrid(Graphics g){
		int cellSize = this.getHeight() / rows;
		g.setColor(Color.red);
		for(int i = 1; i < cols; i++){
			g.drawLine(0,i*cellSize,this.getWidth(),i*cellSize);
		}
		cellSize = this.getWidth() / cols;
		for(int i = 1; i < cols; i++){
			g.drawLine(i*cellSize,0,i*cellSize,this.getHeight());
		}
	}
	
	// Draws all live cells.
	protected void drawLives(Graphics g){
		g.setColor(Color.red);
		for(int i =0; i < rows; i++){
			for(int j = 0; j < cols; j++){
				if ( grid[i][j] ){
					g.fillRect(j*cellWidth,i*cellHeight,cellWidth,cellHeight);
				}
			}
		}
	}
	
	// Returns the number of alive neighbors for the cell at position r,c.  The
	//  grid is finite but unbound, meaning that the neighbor relation wraps around
	//  the grid boundaries.
	protected int nbrOfNeighbors(int r, int c){
		int n = 0;
		int x = 0;
		int y = r-1;
		if ( y < 0 ){ y = rows-1; }
		for (int rCt = 1; rCt <= 3; rCt++){
			x = c-1;
			if ( x < 0 ){ x = cols-1;}
			for(int cCt = 1; cCt <= 3; cCt++){
				if ( x != c || y != r ){
					if ( grid[y][x] ){
						n++;
					}
				}
				x = (x+1) % cols;
			}
			y = (y+1)%rows;
		}
		return n;
	}
	
	// This method implements the rules of the Game of Life.  For each cell,
	//  we simple find the number of neighbors and then bring the cell to life
	//  if appropriate.  
	public void advance(){
		boolean[][] newGrid = new boolean[rows][cols];
		int n =0;
		for(int i = 0; i < rows; i++){
			for(int j = 0; j < cols; j++){
				n = nbrOfNeighbors(i,j);
				if ( grid[i][j] ){
					if ( n == 2 || n == 3 ){
						newGrid[i][j] = true;
					}
					else {
						newGrid[i][j] = false;
					}
				}
				else {
					if ( n == 3){
						newGrid[i][j] = true;
					}
					else {
						newGrid[i][j] = false;
					}
				}
				
			}
		}
		grid = newGrid;
		repaint();
	}
}
