package edu.uwec.cs.wickmr.life.visitor;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/*
 * 
 * This is the main class of the application. 
 */
public class GameOfLifeUI extends JFrame implements GameConstants {
	protected final static Color DEFAULT_LIVE_COLOR = Color.RED;
	protected final static Color DEFAULT_DEAD_COLOR = Color.BLACK;
	protected GameOfLife game;
	protected GameOfLifePanel view;
	protected JPanel contentPane;
	protected JButton advanceButton;
	protected LifeVisitor visitor;

	
	//	Build the appropriate instance of the GameOfLife class and initialize the
	//  application frame.
	public GameOfLifeUI(String title, GameOfLife game, LifeVisitor visitor)  {
		this(title, game, DEFAULT_LIVE_COLOR, DEFAULT_DEAD_COLOR, visitor);
	}
	
	public GameOfLifeUI(String title, GameOfLife game, Color liveColor, Color deadColor, LifeVisitor visitor){
		super(title);
		this.game = game;
		this.view = new GameOfLifePanel(game, liveColor, deadColor);
		this.advanceButton = new JButton("Advance");
		this.contentPane = new JPanel();
		game.attach(this);
		this.visitor = visitor;
		initialize();

	}
	
	// Build the UI and set the "advance" button to cause the game object to 
	//  advance one generation.
	protected void initialize(){
		this.setSize(view.getWidth()+PAD_WIDTH, view.getHeight()+PAD_HEIGHT);
		contentPane.setLayout(new FlowLayout());
		contentPane.add(view);
		contentPane.add(advanceButton);
		
		this.setContentPane(contentPane);
		this.setVisible(true);
		advanceButton.addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent e){
				game.advance(visitor);
			} } );
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}
	
	// Repaint the view to display the changes to the game.
	public void update(){
		repaint();
	}


}
